@@ -29,10 +29,10 @@ compare <- function(x, verbose=TRUE) {
 			cat("-------------------------------------------------------------------------\n")
 		}
 	
-		aL1 <- anovaList(list(cubic=cubic, full=full, IA=IA, additive=additive, diff=diff))
+		aL1 <- anovaList(list(cubic=cubic, full=full, IA=IA, additive=additive, diff=diff, null=null))
 		if (aL1$n.mods > 1) {
 			if (verbose==TRUE) {
-				cat("Stepping down from full polynomial model: Interaction, additive main effects, difference model :\n")
+				cat("Testing directed difference models: Interaction, additive main effects, difference model :\n")
 				cat("-------------------------------------------------------------------------\n")
 			}
 			a1 <- cbind(aL1$ANOVA, ldply(aL1$models, function(X) {
@@ -41,7 +41,10 @@ compare <- function(x, verbose=TRUE) {
 				names(R) <- "R2"
 				n <- nobs(X)
 				k <- free.max - F["df"]
-				R2.p <- pf(((n-k-1)*R)/(k*(1-R)), k, n-k-1, lower.tail=FALSE)
+				
+				R2.p <- ifelse(k==0,
+					NA,
+					pf(((n-k-1)*R)/(k*(1-R)), k, n-k-1, lower.tail=FALSE))
 				names(R2.p) <- "R2.p"
 				return(c(F[c("cfi", "tli", "rmsea", "srmr")], R, R2.p))
 
@@ -54,10 +57,10 @@ compare <- function(x, verbose=TRUE) {
 			res <- a1
 		}
 	
-		aL2 <- anovaList(list(cubic=cubic, full=full, SRSD=SRSD, SSD=SSD, sqdiff=sqdiff))
+		aL2 <- anovaList(list(cubic=cubic, full=full, SRSD=SRSD, SSD=SSD, sqdiff=sqdiff, null=null))
 		if (aL2$n.mods > 1) {
 			if (verbose==TRUE) {
-				cat("\n\nTesting squared differences against full polynomial model:\n")
+				cat("\n\nTesting 'flat ridge' discrepancy models against full polynomial model:\n")
 				cat("-------------------------------------------------------------------------\n")
 			}
 			a2 <- cbind(aL2$ANOVA, ldply(aL2$models, function(X) {
@@ -66,7 +69,35 @@ compare <- function(x, verbose=TRUE) {
 				names(R) <- "R2"
 				n <- nobs(X)
 				k <- free.max - F["df"]
-				R2.p <- pf(((n-k-1)*R)/(k*(1-R)), k, n-k-1, lower.tail=FALSE)
+				R2.p <- ifelse(k==0,
+					NA,
+					pf(((n-k-1)*R)/(k*(1-R)), k, n-k-1, lower.tail=FALSE))
+				names(R2.p) <- "R2.p"
+				return(c(F[c("cfi", "tli", "rmsea", "srmr")], R, R2.p))
+			}))
+			a2 <- a2[, !grepl(".id", colnames(a2))]
+			a2$k <- free.max - a2$Df
+			a2$R2.adj <- 1 - ((1-a2$R2))*((nobs(full)-1)/(nobs(full)-a2$k-1))
+			a2$delta.R2 <- c(NA, a2$R2[1:(nrow(a2)-1)] - a2$R2[2:(nrow(a2))])
+			if (verbose==TRUE) print(round(a2, 3))
+			res <- rbind(res, a2)
+		}
+		
+		aL2b <- anovaList(list(cubic=cubic, full=full, RR=RR, sqdiff=sqdiff, null=null))
+		if (aL2b$n.mods > 1) {
+			if (verbose==TRUE) {
+				cat("\n\nTesting 'rising ridge' against full polynomial model:\n")
+				cat("-------------------------------------------------------------------------\n")
+			}
+			a2 <- cbind(aL2b$ANOVA, ldply(aL2b$models, function(X) {
+				F <- fitmeasures(X)
+				R <- inspect(X, "r2")
+				names(R) <- "R2"
+				n <- nobs(X)
+				k <- free.max - F["df"]
+				R2.p <- ifelse(k==0,
+					NA,
+					pf(((n-k-1)*R)/(k*(1-R)), k, n-k-1, lower.tail=FALSE))
 				names(R2.p) <- "R2.p"
 				return(c(F[c("cfi", "tli", "rmsea", "srmr")], R, R2.p))
 			}))
@@ -109,3 +140,20 @@ compare <- function(x, verbose=TRUE) {
 	invisible(res)
 	})
 }
+
+
+
+
+# compare CFI of intercept-only null model
+
+# CFI2 <- function(x, m1="full", m0="null") {
+# 	cfi2 <- 1-(inspect(x$models[[m1]], "fit")['chisq']-inspect(x$models[[m1]], "fit")['df']) / (inspect(x$models[[m0]], "fit")['chisq']-inspect(x$models[[m0]], "fit")['df'])
+# 	names(cfi2) <- "CFI2"
+# 	return(cfi2)
+# }
+# 
+# TLI2 <- function(x, m1="full", m0="null") {
+# 	tli2 <- ((inspect(x$models[[m0]], "fit")['chisq'] / inspect(x$models[[m0]], "fit")['df']) - (inspect(x$models[[m1]], "fit")['chisq'] / inspect(x$models[[m1]], "fit")['df'])) / ((inspect(x$models[[m0]], "fit")['chisq'] / inspect(x$models[[m0]], "fit")['df']) - 1)
+# 	names(tli2) <- "TLI2"
+# 	return(tli2)
+# }
\ No newline at end of file
